package io.gitlab.jfronny.inceptum.util.api;

import io.gitlab.jfronny.commons.HttpUtils;
import io.gitlab.jfronny.gson.reflect.TypeToken;
import io.gitlab.jfronny.inceptum.model.gitlab.GitlabJob;
import io.gitlab.jfronny.inceptum.model.gitlab.GitlabPackage;
import io.gitlab.jfronny.inceptum.model.gitlab.GitlabPackageFile;
import io.gitlab.jfronny.inceptum.model.gitlab.GitlabProject;

import java.io.IOException;
import java.lang.reflect.Type;
import java.net.URISyntaxException;
import java.util.List;
import java.util.function.Predicate;

public class GitlabApi {
    public static final String PROJECTS = "https://gitlab.com/api/v4/projects/";
    private static final Type packageInfoListType = new TypeToken<List<GitlabPackage>>() {}.getType();
    private static final Type jobListType = new TypeToken<List<GitlabJob>>() {}.getType();
    private static final Type packageFileInfoListType = new TypeToken<List<GitlabPackageFile>>() {}.getType();

    public static GitlabProject getProject(Long projectId) throws IOException, URISyntaxException {
        return HttpUtils.get(PROJECTS + projectId).sendSerialized(GitlabProject.class);
    }

    public static List<GitlabPackage> getPackages(GitlabProject project) throws IOException, URISyntaxException {
        return HttpUtils.get(PROJECTS + project.id + "/packages?order_by=created_at&sort=desc").sendSerialized(packageInfoListType);
    }

    public static List<GitlabJob> getJobs(GitlabProject project, Long pipelineId) throws IOException, URISyntaxException {
        List<GitlabJob> list = HttpUtils.get(PROJECTS + project.id + "/pipelines/" + pipelineId + "/jobs").sendSerialized(jobListType);
        list.sort((left, right) -> right.created_at.compareTo(left.created_at));
        return list;
    }

    public static GitlabPackageFile getFile(GitlabProject project, GitlabPackage packageInfo, Predicate<GitlabPackageFile> isValid) throws IOException, URISyntaxException {
        int page = 0;
        while (true) {
            List<GitlabPackageFile> files = HttpUtils.get(PROJECTS + project.id + "/packages/" + packageInfo.id + "/package_files?per_page=100&page=" + ++page).sendSerialized(packageFileInfoListType);
            if (files.isEmpty()) return null;
            for (GitlabPackageFile file : files) {
                if (isValid.test(file))
                    return file;
            }
        }
    }
}
