package io.gitlab.jfronny.inceptum.util.cache;

import io.gitlab.jfronny.inceptum.util.Utils;

import java.io.Closeable;
import java.io.IOException;
import java.nio.file.*;

import static java.nio.file.StandardWatchEventKinds.*;

public class CachingGsonFile<T> implements Closeable {
    private final Path filePath;
    private final Class<T> type;
    private final WatchService service;
    private T cache = null;

    public CachingGsonFile(Path filePath, Class<T> type) throws IOException {
        this.filePath = filePath;
        this.type = type;
        this.service = FileSystems.getDefault().newWatchService();
        filePath.getParent().register(service, ENTRY_MODIFY);
    }

    public T get() throws IOException {
        WatchKey key = service.poll();
        boolean update = cache == null;
        if (key != null) {
            for (WatchEvent<?> event : key.pollEvents()) {
                update |= event.context().equals(filePath);
            }
            key.reset();
        }
        if (update) cache = Utils.loadObject(filePath, type);
        return cache;
    }

    @Override
    public void close() throws IOException {
        service.close();
    }
}
