package io.gitlab.jfronny.inceptum.util.cache;

import io.gitlab.jfronny.commons.throwable.ThrowingSupplier;
import io.gitlab.jfronny.inceptum.util.Utils;

import java.io.IOException;
import java.lang.reflect.Type;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.concurrent.ConcurrentHashMap;

public class GsonFileCache {
    private final ConcurrentHashMap<String, Object> container = new ConcurrentHashMap<>();
    private final Path cacheDir;

    public GsonFileCache(Path cacheDir) {
        this.cacheDir = cacheDir;
    }

    public void remove(String key) {
        container.remove(key);
        try {
            Files.delete(cacheDir.resolve(key));
        } catch (IOException e) {
            Utils.LOGGER.error("Could not remove cache entry", e);
        }
    }

    public void clear() {
        container.clear();
        try {
            Utils.ls(cacheDir, Files::delete);
        } catch (IOException e) {
            Utils.LOGGER.error("Could not clear cache backend", e);
        }
    }

    public <T, TEx extends Throwable> T get(String key, ThrowingSupplier<T, TEx> builder, Type klazz) throws TEx {
        if (!container.containsKey(key)) {
            Path cd = cacheDir.resolve(key);
            if (Files.exists(cd)) {
                try {
                    T value = Utils.loadObject(cd, klazz);
                    Utils.writeObject(cd, value);
                    container.put(key, value);
                } catch (IOException e) {
                    Utils.LOGGER.error("Could not read cache", e);
                }
            }
            else container.put(key, builder.get());
        }
        //noinspection unchecked
        return (T) container.get(key);
    }
}
