package io.gitlab.jfronny.inceptum.frontend.cli;

import io.gitlab.jfronny.inceptum.model.inceptum.Config;
import io.gitlab.jfronny.inceptum.model.inceptum.InstanceMeta;
import io.gitlab.jfronny.inceptum.util.ConfigHolder;
import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.transport.CredentialsProvider;
import org.eclipse.jgit.transport.UsernamePasswordCredentialsProvider;

import java.nio.file.Path;
import java.util.List;

public abstract class BaseGitCommand extends BaseInstanceCommand {
    public BaseGitCommand(String help, String usage, String... aliases) {
        super(help, usage, aliases);
    }

    public BaseGitCommand(String help, String usage, List<String> aliases, List<Command> subCommands) {
        super(help, usage, aliases, subCommands);
    }

    @Override
    protected void invoke(CommandArgs args, Path instancePath, InstanceMeta meta) throws Exception {
        try (Git git = Git.open(instancePath.toFile())) {
            Config.GitConfig.GitAuth config = ConfigHolder.CONFIG.git.instanceAuths.get(instancePath.getFileName().toString());
            invoke(args.subArgs(), instancePath, meta, git, config.username != null && config.password != null
                    ? new UsernamePasswordCredentialsProvider(config.username, config.password)
                    : CredentialsProvider.getDefault());
        }
    }

    protected abstract void invoke(CommandArgs args, Path instancePath, InstanceMeta meta, Git git, CredentialsProvider credentials) throws Exception;
}
