package io.gitlab.jfronny.inceptum.frontend.cli.commands;

import io.gitlab.jfronny.inceptum.frontend.cli.BaseGitCommand;
import io.gitlab.jfronny.inceptum.frontend.cli.Command;
import io.gitlab.jfronny.inceptum.frontend.cli.CommandArgs;
import io.gitlab.jfronny.inceptum.model.inceptum.InstanceMeta;
import io.gitlab.jfronny.inceptum.util.InstanceExporter;
import io.gitlab.jfronny.inceptum.util.ProcessState;
import io.gitlab.jfronny.inceptum.util.mds.ModsDirScanner;
import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.transport.CredentialsProvider;

import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

//TODO modrinth
public class ExportCommand extends BaseGitCommand {
    public ExportCommand() {
        this(List.of("export"), List.of(
                new MultiMCExportCommand(List.of("multimc", "mmc"), List.of())
        ));
    }

    private ExportCommand(List<String> aliases, List<Command> subCommands) {
        super("Export a CurseForge instance", "<export path>", List.of("export"), subCommands);
    }

    @Override
    protected void invoke(CommandArgs args, Path instancePath, InstanceMeta meta, Git git, CredentialsProvider credentials) throws Exception {
        if (args.length == 0) throw new IllegalAccessException("You must specify a target path");
        if (args.length != 1) throw new IllegalAccessException("Too many arguments");
        ProcessState state = new ProcessState();
        ModsDirScanner mds = ModsDirScanner.get(instancePath.resolve("mods"), meta);
        mds.runOnce((path, mod) -> {});
        InstanceExporter.exportCurseZip(state, instancePath, meta, mds, git, Paths.get(args.get(0)));
    }

    private static class MultiMCExportCommand extends BaseGitCommand {
        public MultiMCExportCommand(List<String> aliases, List<Command> subCommands) {
            super("Export a MultiMC instance", "<export path>", aliases, subCommands);
        }

        @Override
        protected void invoke(CommandArgs args, Path instancePath, InstanceMeta meta, Git git, CredentialsProvider credentials) throws Exception {
            if (args.length == 0) throw new IllegalAccessException("You must specify a target path");
            if (args.length != 1) throw new IllegalAccessException("Too many arguments");
            ProcessState state = new ProcessState();
            ModsDirScanner mds = ModsDirScanner.get(instancePath.resolve("mods"), meta);
            mds.runOnce((path, mod) -> {});
            InstanceExporter.exportMultiMCZip(state, instancePath, meta, mds, git, Paths.get(args.get(0)));
        }
    }
}
