package io.gitlab.jfronny.inceptum.frontend.cli.commands;

import io.gitlab.jfronny.commons.OSUtils;
import io.gitlab.jfronny.inceptum.frontend.cli.Command;
import io.gitlab.jfronny.inceptum.frontend.cli.CommandArgs;
import io.gitlab.jfronny.inceptum.model.inceptum.UpdateInfo;
import io.gitlab.jfronny.inceptum.util.*;

import java.io.IOException;
import java.net.URISyntaxException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;

public class UpdateCheckCommand extends Command {
    private final boolean install;
    public UpdateCheckCommand() {
        super("Checks for inceptum updates", "", List.of("update"), List.of(
                new UpdateCheckCommand("Automatically install updates", "install", true)
        ));
        install = false;
    }

    private UpdateCheckCommand(String help, String name, boolean install) {
        super(help, "", name);
        this.install = install;
    }

    @Override
    protected void invoke(CommandArgs args) {
        if (install && !args.wrapped) {
            Utils.LOGGER.error("Automatic updates are not supported without the wrapper");
            return;
        }
        UpdateInfo updateUrl = getUpdate();
        if (updateUrl == null) {
            Utils.LOGGER.info("No update was found");
        } else {
            if (install) {
                Utils.LOGGER.info("Installing from " + updateUrl);
                try {
                    update(updateUrl, false);
                } catch (IOException | URISyntaxException e) {
                    Utils.LOGGER.error("Could not download update", e);
                }
            }
            else {
                Utils.LOGGER.info("An update was found: " + updateUrl);
            }
        }
    }

    public static UpdateInfo getUpdate() {
        return UpdateChecker.check(ConfigHolder.CONFIG.channel, MetaHolder.VERSION.version, MetaHolder.VERSION.flavor, channel -> {
            Utils.LOGGER.error("No stable version was found, switching to experimental channel");
            ConfigHolder.CONFIG.channel = channel;
            ConfigHolder.saveConfig();
        });
    }

    public static void update(UpdateInfo source, boolean relaunch) throws IOException, URISyntaxException {
        Utils.LOGGER.info("Downloading " + source.url());
        Path jarPath = MetaHolder.LIBRARIES_DIR.resolve("io/gitlab/jfronny/inceptum/Inceptum")
                .resolve(source.newVersion().toString())
                .resolve("Inceptum-" + source.newVersion() + '-' + OSUtils.TYPE.getMojName() + ".jar")
                .toAbsolutePath();
        Files.createDirectories(jarPath.getParent());
        Utils.downloadFile(source.url(), source.sha1(), jarPath);
        if (relaunch) {
            Runtime.getRuntime().addShutdownHook(new Thread(() -> {
                try {
                    new ProcessBuilder(OSUtils.getJvmBinary(),
                            "-jar",
                            jarPath.toString())
                            .inheritIO()
                            .start();
                } catch (IOException e) {
                    Utils.LOGGER.error("Could not relaunch", e);
                }
            }));
        }
    }
}
