package io.gitlab.jfronny.inceptum.frontend.gui.window;

import imgui.ImGui;
import imgui.flag.ImGuiWindowFlags;
import imgui.type.ImBoolean;
import imgui.type.ImInt;
import io.gitlab.jfronny.inceptum.Inceptum;
import io.gitlab.jfronny.inceptum.InceptumGui;
import io.gitlab.jfronny.inceptum.frontend.gui.control.InstanceView;
import io.gitlab.jfronny.inceptum.util.ConfigHolder;
import io.gitlab.jfronny.inceptum.util.MapAppender;
import io.gitlab.jfronny.inceptum.util.MetaHolder;
import io.gitlab.jfronny.inceptum.util.Utils;
import io.gitlab.jfronny.inceptum.util.api.account.AccountManager;
import io.gitlab.jfronny.inceptum.util.api.account.AuthInfo;
import io.gitlab.jfronny.inceptum.util.api.account.MicrosoftAccount;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;
import java.util.function.Predicate;

public class MainWindow extends Window {
    private final ImBoolean darkTheme = new ImBoolean(ConfigHolder.CONFIG.darkTheme);
    private final ImBoolean debugTools = new ImBoolean(false);
    private final ImInt accountIndex = new ImInt(AccountManager.getSelectedIndex());
    public MainWindow() {
        super("Inceptum");
    }

    @Override
    public int getFlags() {
        return ImGuiWindowFlags.MenuBar | ImGuiWindowFlags.AlwaysAutoResize;
    }

    @Override
    public void draw() {
        ImGui.beginMenuBar();
        if (ImGui.beginMenu("File")) {
            if (ImGui.menuItem("New Instance")) InceptumGui.open(new NewInstanceWindow());
            if (ImGui.menuItem("Re-download resources")) {
                try {
                    Utils.clearDirectory(MetaHolder.ASSETS_DIR);
                    Utils.clearDirectory(MetaHolder.LIBRARIES_DIR, path -> !path.startsWith(MetaHolder.LIBRARIES_DIR.resolve("io/gitlab/jfronny")));
                    Utils.clearDirectory(MetaHolder.NATIVES_DIR, path -> !path.endsWith("forceload"));
                    Utils.clearDirectory(MetaHolder.CACHE_DIR);
                    GuiUtil.reload();
                } catch (IOException e) {
                    Inceptum.showError("Could not execute refresh task", e);
                }
            }
            if (ImGui.menuItem("Exit Inceptum")) InceptumGui.exit();
            ImGui.endMenu();
        }
        if (ImGui.beginMenu("Account")) {
            if (ImGui.menuItem("New")) InceptumGui.open(new MicrosoftLoginWindow());
            AuthInfo selected = AccountManager.getSelectedAccount();
            List<MicrosoftAccount> accounts = AccountManager.getAccounts();
            int accountsSize = accounts.size();
            for (int i = 0; i < accountsSize; i++) {
                MicrosoftAccount account = accounts.get(i);
                if (selected.equals(account)) accountIndex.set(i);
                if (ImGui.radioButton(account.minecraftUsername, accountIndex, i)) {
                    AccountManager.switchAccount(account);
                }
                ImGui.sameLine();
                if (ImGui.button("X")) AccountManager.removeAccount(account);
            }
            if (ImGui.radioButton("None", accountIndex, accountsSize)) {
                AccountManager.switchAccount(null);
            }
            ImGui.endMenu();
        }
        if (ImGui.beginMenu("Settings")) {
            if (ImGui.checkbox("Dark Theme", darkTheme)) {
                ConfigHolder.CONFIG.darkTheme = darkTheme.get();
                ConfigHolder.saveConfig();
                InceptumGui.applyTheme();
            }
            ImGui.endMenu();
        }
        if (ImGui.beginMenu("Help")) {
            if (ImGui.menuItem("About")) InceptumGui.open(new AboutWindow());
            if (ImGui.menuItem("Log")) InceptumGui.open(new LogWindow(MapAppender.LOG));
            ImGui.checkbox("Debug Tools", debugTools);
            ImGui.endMenu();
        }
        ImGui.endMenuBar();

        if (debugTools.get()) {
            ImGui.showDemoWindow();
        }

        List<Path> paths;
        try {
            if (!Files.exists(MetaHolder.INSTANCE_DIR)) Files.createDirectories(MetaHolder.INSTANCE_DIR);
            paths = Utils.ls(MetaHolder.INSTANCE_DIR, (Predicate<Path>) Files::isDirectory);
        } catch (IOException e) {
            Utils.LOGGER.error("Could not list instances");
            return;
        }
        if (paths.isEmpty()) {
            ImGui.text("You have not yet created an instance");
            ImGui.text("Use File->New Instance to do so");
        }
        else InstanceView.draw(paths);
    }
}
