package io.gitlab.jfronny.inceptum.frontend.gui.window.edit;

import imgui.ImGui;
import io.gitlab.jfronny.commons.throwable.ThrowingBiConsumer;
import io.gitlab.jfronny.inceptum.Inceptum;
import io.gitlab.jfronny.inceptum.InceptumGui;
import io.gitlab.jfronny.inceptum.frontend.gui.control.Tab;
import io.gitlab.jfronny.inceptum.frontend.gui.window.dialog.ProcessStateWatcherWindow;
import io.gitlab.jfronny.inceptum.util.InstanceExporter;
import io.gitlab.jfronny.inceptum.util.ProcessState;
import org.lwjgl.PointerBuffer;
import org.lwjgl.system.MemoryStack;
import org.lwjgl.util.tinyfd.TinyFileDialogs;

import java.nio.file.Path;
import java.nio.file.Paths;

public class ExportTab extends Tab {
    private final InstanceEditWindow window;

    public ExportTab(InstanceEditWindow window) {
        super("Export");
        this.window = window;
    }

    @Override
    protected void renderInner() {
        if (window.mds.isComplete()) {
            exportVariant("CurseForge", "zip", (state, exportPath) -> {
                InstanceExporter.exportCurseZip(state, window.path, window.instance, window.mds, window.git, exportPath);
            });
            exportVariant("MultiMC", "zip", (state, exportPath) -> {
                InstanceExporter.exportMultiMCZip(state, window.path, window.instance, window.mds, window.git, exportPath);
            });
        }
        else {
            ImGui.text("The mods directory scan must be completed.\nThe progress for this can be observed in the mods tab");
        }
    }

    private <T extends Throwable> void exportVariant(String name, String packExt, ThrowingBiConsumer<ProcessState, Path, T> export) {
        if (ImGui.button(name)) {
            try (MemoryStack stack = MemoryStack.stackPush()) {
                PointerBuffer aFilterPatterns = stack.mallocPointer(2);
                aFilterPatterns.put(stack.UTF8("*." + packExt));
                aFilterPatterns.flip();
                String p = TinyFileDialogs.tinyfd_saveFileDialog("Export Pack", "", aFilterPatterns, name + " packs (*." + packExt + ")");
                if (p != null) {
                    ProcessState state = new ProcessState(InstanceExporter.STEP_COUNT, "Initializing...");
                    InceptumGui.open(new ProcessStateWatcherWindow("Exporting", "Could not export pack", state, cToken -> {
                        Path exportPath = Paths.get(p);
                        export.accept(state, exportPath);
                        Inceptum.showInfo(window.path.getFileName().toString() + " has been successfully exported to " + exportPath, "Successfully exported");
                    }, null));
                }
            }
        }
    }
}
