package io.gitlab.jfronny.inceptum.util;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;

public class InstanceLock {
    private static final String INCEPTUM_LOCK = "inceptum.lock";
    private static final String INCEPTUM_SETUP_LOCK = "inceptum.setup.lock";
    public static void setSetupLock(Path instancePath, boolean state) throws IOException {
        if (isSetupLocked(instancePath)) {
            if (!state) Files.delete(instancePath.resolve(INCEPTUM_SETUP_LOCK));
        }
        else {
            if (state) Files.createDirectories(instancePath.resolve(INCEPTUM_SETUP_LOCK));
        }
    }

    public static boolean isSetupLocked(Path instancePath) {
        return Files.exists(instancePath.resolve(INCEPTUM_SETUP_LOCK));
    }

    public static void setRunningLock(Path instancePath, long pid) throws IOException {
        Files.writeString(instancePath.resolve(INCEPTUM_LOCK), Long.toString(pid));
    }

    public static boolean isRunningLocked(Path instancePath) throws IOException {
        if (!Files.exists(instancePath.resolve(INCEPTUM_LOCK))) return false;
        if (ProcessUtils.isProcessAlive(Files.readString(instancePath.resolve(INCEPTUM_LOCK))))
            return true;
        Files.delete(instancePath.resolve(INCEPTUM_LOCK));
        return false;
    }

    public static boolean isLocked(Path instancePath) throws IOException {
        return isSetupLocked(instancePath) || isRunningLocked(instancePath);
    }
}
