package io.gitlab.jfronny.inceptum.util.api;

import io.gitlab.jfronny.gson.reflect.TypeToken;
import io.gitlab.jfronny.inceptum.model.modrinth.ModrinthProject;
import io.gitlab.jfronny.inceptum.model.modrinth.ModrinthProjectType;
import io.gitlab.jfronny.inceptum.model.modrinth.ModrinthSearchResult;
import io.gitlab.jfronny.inceptum.model.modrinth.ModrinthVersion;
import io.gitlab.jfronny.inceptum.util.Utils;

import java.io.IOException;
import java.lang.reflect.Type;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

public class ModrinthApi {
    private static final String API_HOST = "https://api.modrinth.com/";
    private static final int ITEMS_PER_PAGE = 20;
    private static final Type modrinthVersionListType = new TypeToken<List<ModrinthVersion>>() {}.getType();

    //TODO search by categories: facets:[["versions:$ver","versions:$ver"],["categories:$cat","categories:$cat"]]
    //TODO filter server/client-only mods
    public static ModrinthSearchResult search(String query, int page, String version, ModrinthProjectType type) throws IOException {
        return Utils.downloadObject(Utils.buildUrl(API_HOST, "v2/search", Map.of(
                "query", query,
                "facets", "[[\"versions:" + version + "\"],[\"categories:fabric\"],[\"project_type:" + type + "\"]]",
                "index", "relevance",
                "offset", Integer.toString(page * ITEMS_PER_PAGE),
                "limit", Integer.toString(ITEMS_PER_PAGE)
        )), ModrinthSearchResult.class);
    }

    public static ModrinthProject getMod(String id) throws IOException {
        return Utils.downloadObject(API_HOST + "v2/project/" + id, ModrinthProject.class);
    }

    public static List<ModrinthVersion> getVersions(String mod) throws IOException {
        List<ModrinthVersion> versions = Utils.downloadObject(API_HOST + "v2/project/" + mod + "/version", modrinthVersionListType);
        versions.sort(Comparator.comparing(version -> version.date_published));
        return versions;
    }

    public static ModrinthVersion getVersion(String id) throws IOException {
        return Utils.downloadObject(API_HOST + "v2/version/" + id, ModrinthVersion.class);
    }

    public static ModrinthVersion getVersionByHash(String sha1) throws IOException {
        return Utils.downloadObject(API_HOST + "v2/version_file/" + sha1 + "?algorithm=sha1", ModrinthVersion.class);
    }
}
