package io.gitlab.jfronny.inceptum.util.install;

import io.gitlab.jfronny.inceptum.model.inceptum.InstanceMeta;
import io.gitlab.jfronny.inceptum.model.inceptum.LoaderInfo;
import io.gitlab.jfronny.inceptum.model.mojang.VersionInfo;
import io.gitlab.jfronny.inceptum.model.mojang.VersionsListInfo;
import io.gitlab.jfronny.inceptum.util.InstanceLock;
import io.gitlab.jfronny.inceptum.util.ProcessState;
import io.gitlab.jfronny.inceptum.util.Utils;
import io.gitlab.jfronny.inceptum.util.api.FabricMetaApi;
import io.gitlab.jfronny.inceptum.util.api.McApi;
import io.gitlab.jfronny.inceptum.util.install.steps.*;

import java.io.IOException;
import java.nio.file.Path;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.atomic.AtomicBoolean;

public class Steps {
    public static Set<Step> STEPS = new LinkedHashSet<>(List.of(
            new SetupDirsStep(),
            new DownloadJavaStep(),
            new DownloadClientStep(),
            new DownloadLibrariesStep(),
            new DownloadAssetsStep(),
            new WriteMetadataStep(),
            new RunMdsStep()
    ));

    public static ProcessState createProcessState() {
        return new ProcessState(STEPS.size(), "Initializing");
    }

    public static void reDownload(Path instance, ProcessState state) throws IOException {
        if (InstanceLock.isLocked(instance)) return;
        InstanceMeta im = Utils.loadObject(instance.resolve("instance.json"), InstanceMeta.class);
        boolean found = false;
        for (VersionsListInfo version : McApi.getVersions().versions) {
            if (version.id.equals(im.getMinecraftVersion())) {
                found = true;
                VersionInfo vi = McApi.getVersionInfo(version);
                if (im.isFabric()) vi = FabricMetaApi.addFabric(vi, im.getLoaderVersion(), FabricMetaApi.FabricVersionInfoType.Both);
                LoaderInfo li = im.isFabric()
                        ? new LoaderInfo(LoaderInfo.Type.Fabric, im.getLoaderVersion())
                        : LoaderInfo.NONE;
                SetupStepInfo info = new SetupStepInfo(vi, li, instance.getFileName().toString(), state);
                for (Step step : Steps.STEPS) {
                    state.incrementStep("Starting " + step.getClass().getSimpleName());
                    step.execute(info, new AtomicBoolean(false));
                }
            }
        }
        if (!found) throw new IOException("Could not identify minecraft version " + im.getMinecraftVersion());
    }
}
