package io.gitlab.jfronny.inceptum.util.install.steps;

import io.gitlab.jfronny.inceptum.util.install.SetupStepInfo;
import io.gitlab.jfronny.inceptum.util.install.Step;
import io.gitlab.jfronny.inceptum.model.mojang.JvmFileInfo;
import io.gitlab.jfronny.inceptum.model.mojang.MojangFileDownload;
import io.gitlab.jfronny.inceptum.model.mojang.VersionInfo;
import io.gitlab.jfronny.inceptum.util.MetaHolder;
import io.gitlab.jfronny.inceptum.util.Utils;
import io.gitlab.jfronny.inceptum.util.api.McApi;

import java.io.IOException;
import java.net.URISyntaxException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;

public class DownloadJavaStep implements Step {
    @Override
    public void execute(SetupStepInfo info, AtomicBoolean stopThread) throws IOException {
        VersionInfo.JavaVersion ver = info.version().javaVersion;
        Path jvmDir = MetaHolder.NATIVES_DIR.resolve(ver.component).resolve(Integer.toString(ver.majorVersion));
        if (Files.exists(jvmDir)) return;
        info.setState("Downloading JVM");
        Files.createDirectories(jvmDir);
        for (Map.Entry<String, JvmFileInfo.File> entry : McApi.getJvm(ver.component, ver.majorVersion).entrySet()) {
            Path tPath = jvmDir.resolve(entry.getKey());
            switch (entry.getValue().type) {
                case "file" -> {
                    MojangFileDownload mf = entry.getValue().downloads.raw;
                    info.setState("jvm: Downloading " + tPath);
                    try {
                        Utils.downloadFile(mf.url, mf.sha1, tPath);
                    } catch (URISyntaxException e) {
                        throw new IOException("Could not download jvm", e);
                    }
                    if (entry.getValue().executable) {
                        if (!tPath.toFile().setExecutable(true))
                            info.setState("Could not set executable bit for " + tPath);
                    }
                }
                case "directory" -> Files.createDirectories(tPath);
            }
        }
        if (Files.exists(MetaHolder.FORCE_LOAD_PATH)) {
            Utils.copyContent(MetaHolder.FORCE_LOAD_PATH, jvmDir.resolve("bin"));
        }
    }
}
