package io.gitlab.jfronny.inceptum.util.install.steps;

import io.gitlab.jfronny.inceptum.util.ProcessState;
import io.gitlab.jfronny.inceptum.util.install.SetupStepInfo;
import io.gitlab.jfronny.inceptum.util.install.Step;
import io.gitlab.jfronny.inceptum.model.inceptum.ArtifactInfo;
import io.gitlab.jfronny.inceptum.model.inceptum.InstanceMeta;
import io.gitlab.jfronny.inceptum.model.mojang.VersionInfo;
import io.gitlab.jfronny.inceptum.util.MetaHolder;
import io.gitlab.jfronny.inceptum.util.Utils;
import io.gitlab.jfronny.inceptum.util.VersionInfoLibraryResolver;

import java.io.IOException;
import java.net.URISyntaxException;
import java.nio.file.FileSystem;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.concurrent.atomic.AtomicBoolean;

public class DownloadLibrariesStep implements Step {
    @Override
    public void execute(SetupStepInfo info, AtomicBoolean stopThread) throws IOException {
        execute(info.version(), stopThread, info.currentState());
    }

    public static void execute(VersionInfo version, AtomicBoolean stopThread, ProcessState currentState) throws IOException {
        for (ArtifactInfo artifact : VersionInfoLibraryResolver.getRelevant(version)) {
            if (stopThread.get()) return;
            Path path = MetaHolder.LIBRARIES_DIR.resolve(artifact.path);
            if (!Files.exists(path)) {
                currentState.updateStep("Downloading library: " + artifact.path);
                if (!Files.exists(path.getParent())) Files.createDirectories(path.getParent());
                if (!artifact.url.endsWith(".jar")) {
                    Utils.LOGGER.info("Not a valid URL for a jar: " + artifact.url);
                    continue;
                }
                try {
                    Utils.downloadFile(artifact.url, artifact.sha1, path);
                } catch (URISyntaxException e) {
                    throw new IOException("Could not download library", e);
                }
            }
            if (artifact.isNative) {
                currentState.updateStep("Extracting natives");
                try (FileSystem libFs = Utils.openZipFile(path, false)) {
                    Utils.copyContent(libFs.getPath("."), MetaHolder.NATIVES_DIR.resolve(InstanceMeta.getMinecraftVersion(version.id)));
                }
                catch (Throwable t) {
                    Files.delete(path);
                    throw new IOException("Could not extract native", t);
                }
            }
        }
    }
}
