package io.gitlab.jfronny.inceptum;

import io.gitlab.jfronny.commons.*;
import io.gitlab.jfronny.inceptum.model.inceptum.*;
import io.gitlab.jfronny.inceptum.util.*;

import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.nio.file.*;
import java.util.*;
import java.util.stream.*;

public class Wrapper {
    private static final Path INCEPTUM_LIB_DIR = MetaHolder.BASE_PATH.resolve("libraries/io/gitlab/jfronny/inceptum/Inceptum");

    public static void main(String[] args) throws IOException, ClassNotFoundException, NoSuchMethodException, InvocationTargetException, IllegalAccessException, URISyntaxException, NoSuchFieldException {
        Utils.setSystemLoader(WrapperStrap.SYSTEM_LOADER);
        MetaHolder.setWrapperFlag();
        System.out.println("Inceptum Wrapper v" + MetaHolder.VERSION.version + " (" + MetaHolder.VERSION.flavor + ")");
        InceptumEnvironmentInitializer.initialize();
        if (!Files.exists(INCEPTUM_LIB_DIR)) downloadLatest();
        Optional<Path> localBinary = selectLocalBinary();
        if (localBinary.isEmpty()) {
            downloadLatest();
            localBinary = selectLocalBinary();
        }
        if (localBinary.isEmpty()) {
            throw new FileNotFoundException("Could not identify or download a valid binary");
        }
        String[] newArgs = new String[args.length + 1];
        newArgs[0] = "wrapper";
        System.arraycopy(args, 0, newArgs, 1, args.length);
        System.out.println("Starting Inceptum ClassLoader");
        WrapperStrap.switchEnv(localBinary.get(),
                "io.gitlab.jfronny.inceptum.Inceptum",
                newArgs);
    }

    private static void downloadLatest() throws IOException {
        System.err.println("No Inceptum jar was identified");
        UpdateInfo ui = UpdateChecker.check(UpdateChannel.Stable, new ComparableVersion("0"), OSUtils.TYPE.getMojName(), c -> {});
        if (ui == null) {
            throw new FileNotFoundException("Could not identify a valid inceptum version. Are you connected to the internet?");
        }
        System.err.println("Downloading " + ui.url());
        Path dir = INCEPTUM_LIB_DIR.resolve(ui.newVersion().toString());
        Files.createDirectories(dir);
        try (InputStream is = new URL(ui.url()).openStream()) {
            Files.write(dir.resolve("Inceptum-" + ui.newVersion() + '-' + OSUtils.TYPE.getMojName() + ".jar"), is.readAllBytes());
        }
        catch (Throwable t) {
            Files.delete(dir);
        }
    }

    private static Optional<Path> selectLocalBinary() throws IOException {
        Path pathChosen = null;
        ComparableVersion versionChosen = new ComparableVersion("0");
        try (Stream<Path> inceptumLibVersionDirStream = Files.list(INCEPTUM_LIB_DIR)) {
            for (Path inceptumVersionPath : inceptumLibVersionDirStream.toList()) {
                ComparableVersion versionCurrent = new ComparableVersion(inceptumVersionPath.getFileName().toString());
                if (versionCurrent.compareTo(versionChosen) > 0) {
                    inceptumVersionPath = inceptumVersionPath.resolve("Inceptum-" + versionCurrent + '-' + OSUtils.TYPE.getMojName() + ".jar");
                    if (Files.exists(inceptumVersionPath)) {
                        versionChosen = versionCurrent;
                        pathChosen = inceptumVersionPath;
                    }
                    else {
                        Utils.LOGGER.error("Candidate " + inceptumVersionPath + " failed: doesn't exist");
                    }
                }
            }
        }
        if (pathChosen == null) {
            Utils.LOGGER.error("Something went wrong, please try again");
            return Optional.empty();
        }
        return Optional.of(pathChosen);
    }
}
