package io.gitlab.jfronny.inceptum;

import io.gitlab.jfronny.inceptum.util.Utils;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;

public class WrapperClassLoader extends ClassLoader {
    private final Path source;

    public WrapperClassLoader(Path zipArchive) throws IOException, URISyntaxException {
        super(null);
        this.source = Files.isDirectory(zipArchive) ? zipArchive : Utils.openZipFile(zipArchive, false).getPath("");
    }

    @Override
    protected Class<?> findClass(String name) throws ClassNotFoundException {
        Path path = getPath(name);
        if (!Files.exists(path)) return WrapperStrap.SYSTEM_LOADER.loadClass(name);
        byte[] clazz;
        try {
            clazz = Files.readAllBytes(path);
        } catch (IOException e) {
            throw new ClassNotFoundException("Could not find class", e);
        }
        return defineClass(name, clazz, 0, clazz.length);
    }

    @Override
    protected Class<?> loadClass(String name, boolean resolve) throws ClassNotFoundException {
        if (!Files.exists(getPath(name)) || name.equals("io.gitlab.jfronny.inceptum.WrapperStrap")) return WrapperStrap.SYSTEM_LOADER.loadClass(name);
        synchronized(this.getClassLoadingLock(name)) {
            Class<?> c = this.findLoadedClass(name);
            if (c == null) c = this.findClass(name);
            if (resolve) this.resolveClass(c);
            return c;
        }
    }

    @Override
    public URL getResource(String name) {
        Path p = source.resolve(name);
        if (!Files.exists(p)) return WrapperStrap.SYSTEM_LOADER.getResource(name);
        try {
            return p.toUri().toURL();
        } catch (MalformedURLException e) {
            throw new RuntimeException("Could not get resource", e);
        }
    }

    private Path getPath(String className) {
        return source.resolve(className.replace('.', '/') + ".class");
    }
}
